@@ -21,7 +21,7 @@ use MIME::Base64;
 use File::Glob;
 use Hash::Util qw( lock_hashref unlock_hashref lock_ref_keys );
 
-our $VERSION = '1.43';
+our $VERSION = '1.44';
 
 our @ISA    = qw( Exporter );
 our @EXPORT = qw(
@@ -106,6 +106,9 @@ our @EXPORT = qw(
               str_pad
               str_pad_center
               str_countable
+
+              str_kmg_to_num
+              str_hms_to_secs
               
               str_password_strength
 
@@ -123,10 +126,31 @@ our @EXPORT = qw(
               
               glob_tree
               read_dir_entries
+
               fftwalk
+                  
+                  FFT_FILES
+                  FFT_DIRS
+                  
+                  FFT_SYMF
+                  FFT_SYMD
+                  
+                  FFT_FOLLOW
+
+                  FFT_ALL
+                  FFT_ALL4
+                  FFT_FULL
               
               ref_freeze
               ref_thaw
+
+              int2hex
+              hex2int
+              
+              bcd2int
+              int2bcd
+              bcd2str
+
             );
 
 our %EXPORT_TAGS = (
@@ -605,6 +629,32 @@ sub str_countable
   return $count == 0 ? $many : $count == 1 ? $one : $many;
 }
 
+sub str_kmg_to_num
+{
+  my $s = uc shift;
+  return undef unless $s =~ /^\s*(\d+(\.\d*)?)(\s*([KMGTP]))?/;
+  return $1 unless $4;
+  return $1 * ( 1024 ** ( index( 'KMGTP', $4 ) + 1 ) );
+}
+
+sub str_hms_to_secs
+{
+  my $s = uc shift;
+  
+  my $secs;
+  $s .= 's' if $s =~ /^[\s\d]+$/;
+  while( $s =~ /(\d+)\s*([WDHMS])/gi )
+    {
+    if   ( lc $2 eq 's' ) { $secs += $1; }
+    elsif( lc $2 eq 'm' ) { $secs += $1 * 60; }
+    elsif( lc $2 eq 'h' ) { $secs += $1 * 60 * 60; }
+    elsif( lc $2 eq 'd' ) { $secs += $1 * 60 * 60 * 24; }
+    elsif( lc $2 eq 'w' ) { $secs += $1 * 60 * 60 * 24 * 7; }
+    }
+  
+  return $secs;
+}
+
 ##############################################################################
 # str_password_strength()
 # returns a number representing password strength
@@ -1115,11 +1165,26 @@ sub read_dir_entries
 
 ##############################################################################
 
+use constant 
+{
+    FFT_FILES  => 0x01,
+    FFT_DIRS   => 0x02,
+    
+    FFT_SYMF   => 0x04, # allow symlink files in result (requires FFT_FILES)
+    FFT_SYMD   => 0x08, # allow symlink dirs in result  (requires FFT_DIRS )
+    
+    FFT_FOLLOW => 0x10,
+
+    FFT_ALL    => 0x01 | 0x02,
+    FFT_ALL4   => 0x01 | 0x02 | 0x04 | 0x08,
+    FFT_FULL   => 0x01 | 0x02 | 0x04 | 0x08 | 0x10,
+};
+
 sub __fftwalk
 {
-  my $e = shift;
-  my $a = shift;
-  my $f = shift; # filter: 0 all, 1 files, 2 dirs
+  my $e  = shift;
+  my $a  = shift;
+  my $ty = shift; # typemap, see FFTs above
   
   opendir( my $dir, $e ) or return undef;
   my $ee;
@@ -1128,41 +1193,62 @@ sub __fftwalk
     next if $ee eq '.' or $ee eq '..';
     my $eee = "$e/$ee";
     
-    next if -l $eee; # FIXME: TODO: OPTION!!!!!!!!!
-    
-    if( -d $eee )
+    my $is_dir  = -d $eee;
+    my $is_link = -l $eee;
+
+    if( $is_dir )
       {
-      push @$a, $eee if $f != 1;
-      __fftwalk( $eee, $a, $f );
+      push @$a, $eee if $is_link ? $ty & FFT_DIRS && $ty & FFT_SYMD : $ty & FFT_DIRS;
+      __fftwalk( $eee, $a, $ty ) if ! $is_link or $ty & FFT_FOLLOW;
       }
     else
       {
-      push @$a, $eee if $f != 2;
+      push @$a, $eee if $is_link ? $ty & FFT_FILES && $ty & FFT_SYMF : $ty & FFT_FILES;
       }  
     }
   closedir( $dir );
 }
 
 # fast file tree walk
-# first argument can be options hash and is optional
+# first argument traversal typemap scalar or hash with options
 # rest of arguments are directory names to be walked
 # options hash can have:
-# ARRAY => hashref_for_result_list
-# MODE  => ALL   or 0 to scan all files and dirs
-# MODE  => FILES or 1 to scan files only
-# MODE  => DIRS  or 2 to scan directories only
+#   TYPE  => typemap
+# this option tells which types of filesystem entries to be processed:
+#   FFT_FILES  -- add found files
+#   FFT_DIRS   -- add found directories
+#   FFT_SYMF   -- add found file symlinks (needs FFT_FILES)
+#   FFT_SYMD   -- add found dir  symlinks (needs FFT_DIRS )
+#   FFT_FOLLOW -- follow/traverse symlink dirs
+# there are few shortcut options:
+#   FFT_ALL    -- all files and dirs but no symlinks
+#   FFT_ALL4   -- all files and dirs including symlinks
+#   FFT_FULL   -- all files, dirs, symlinks and follow symlink dirs
+# if TYPE is zero, fftwalk will not do anything
+#   ARRAY => hashref_for_result_list
+
 sub fftwalk
 {
-  my $opt = hash_uc( ref( $_[0] ) eq 'HASH' ? shift : {} );
+  my $ty = shift;
+
+  my $opt = {};
+  if( ref( $ty ) eq 'HASH' )
+    {
+    $opt = $ty;
+    $ty = $opt->{ 'TYPE' };
+    }
+  else
+    {
+    $opt = {};
+    }  
   
-  my $f;
-  $f = 0 if $opt->{ 'MODE' } =~ /^(A(LL)?|0|\*|FD|DF)$/i;
-  $f = 1 if $opt->{ 'MODE' } =~ /^(F(ILES)?|1)$/i;
-  $f = 2 if $opt->{ 'MODE' } =~ /^(D(IRS)?|2)$/i;
+  die "fftwalk() uses TYPE instead of MODE" if $opt->{ 'MODE' };
 
   my $e = $opt->{ 'ARRAY' } ? $opt->{ 'ARRAY' } : [];
 
-  __fftwalk( $_, $e, $f ) for @_;
+  return $e unless $ty > 0; # do nothing if TYPE is zero
+
+  __fftwalk( $_, $e, $ty ) for @_;
   return $e;
 }
 
@@ -1188,6 +1274,62 @@ sub ref_thaw
 
 ##############################################################################
 
+sub int2hex
+{
+  return sprintf( "%X", shift );
+}
+
+*hex2int = *CORE::hex;
+
+##############################################################################
+
+sub bcd2int
+{
+  my $bcd = shift;
+  
+  my $int = 0;
+
+  my @bcd = unpack 'C*', $bcd;
+  my $p = @bcd * 2 - 1;
+  for( @bcd )
+    {
+    $int += ( 10 ** $p-- ) * ( ( $_ & 0xF0 ) >> 4 );
+    $int += ( 10 ** $p-- ) * ( ( $_ & 0x0F )      );
+    }
+
+  return $int;
+}
+
+sub int2bcd
+{
+  my $int = shift;
+  my $len = shift; # in how many bytes to produce bcd
+  
+  die "int2bcd() is not yet implemented";
+}
+
+sub bcd2str
+{
+  my $bcd = shift;
+  
+  my $str;
+
+  my @bcd = unpack 'C*', $bcd;
+  for( @bcd )
+    {
+    $str .= ( ( $_ & 0xF0 ) >> 4 );
+    $str .= ( ( $_ & 0x0F )      );
+    }
+
+  return $str;
+}
+
+##############################################################################
+
+# sub format_ascii_table
+
+##############################################################################
+
 BEGIN { __url_escapes_init(); }
 INIT  { __url_escapes_init(); }
 
@@ -1345,6 +1487,10 @@ INIT  { __url_escapes_init(); }
                       # returns 'day'  for $dc == 1
                       # returns 'days' for $dc >  1
 
+  my $num = str_kmg_to_num(   '1K' ); # returns 1024   
+  my $num = str_kmg_to_num( '2.5M' ); # returns 2621440
+  my $num = str_kmg_to_num(   '1T' ); # returns 1099511627776
+
   # --------------------------------------------------------------------------
 
   # find all *.txt files in all subdirectories starting from /usr/local
@@ -1354,6 +1500,12 @@ INIT  { __url_escapes_init(); }
   # read directory entries names (without full paths)
   my @files_and_dirs = read_dir_entries( '/tmp/secret/dir' );
 
+  # --------------------------------------------------------------------------
+
+  my $int   = bcd2int( $bcd_bytes ); # convert BCD byte data to integer
+  my $bytes = int2bcd( $int );       # convert integer to BCD bytes
+  my $str   = bcd2str( $bcd_bytes ); # convert BCD byte data to string
+
 =head1 FUNCTIONS
 
 =head2 hash_validate( $data_hr, $validate_hr );
